import os.path
import json
from termcolor import colored, cprint
from rcon.source import Client

# Declare variables that are needed everywhere.

servers_file = "./.midnight_servers.json"
servers = []

# Loads servers in from the servers_file, if it does not exist, trigger adding first server via create_servers.
def load_servers():
    global servers
    if os.path.isfile(servers_file):
        with open(servers_file,'r') as openfile:
            servers = json.load(openfile)
    else:
        print(colored('\n!! Looks like you do not have any servers yet - Let\'s add one.','white','on_green',attrs=['bold']))
        create_server()

# Write the contents to the servers variable to the servers_file.
def write_servers():
    json_obj = json.dumps(servers,indent=4) # Add indentation when dumping to string.
    with open(servers_file, "w") as outfile:
        outfile.write(json_obj)

# Lists servers, default action is to list all information minus password. Second option lists only ID and name.
def list_servers(mode=int(1)): # make sure a default is set.
    output = ""
    if(mode==1):
        index = int(0)
        for x in servers: # for each dict in the servers array as x.
            strindex = str(index)
            output+='ID: '+strindex+' ===\n    Name:'+x['name']+'\n    Address:'+x['address']+'\n    Port:'+x['port']+'\n'
            index += 1
    elif(mode==2):
        index = int(0)
        for x in servers:
            strindex = str(index)
            output+='ID: '+strindex+' ('+x['name']+')\n'
            index += 1
    return output

# Creates new server dicts in the server array.
def create_server():
    global servers # Global so that we can write to it.
    print(colored('\n=== Add Server ===','green'))
    srv_name = input(colored('Name: ','yellow'))
    srv_addr = input(colored('Address: ','yellow'))
    srv_port = input(colored('Port: ','yellow'))
    srv_pass = input(colored('Password: ','yellow'))
    print(colored('OK\n','green'))
    new_server = [{"name":srv_name,"address":srv_addr,"port":srv_port,"password":srv_pass}] # Build new dict in array.
    servers = servers + new_server # Append new array to existing array.
    write_servers() # Write to the file.

# Used to delete servers from the array and write.
def pop_server():
    global servers # Global so we can write
    print(colored('\n=== Remove Server ===\nPick a server to delete','red'))
    print(colored(list_servers(2),'light_blue')) # Lists server ID and Name only.
    delete = input(colored('(enter to cancel) ?: ','yellow'))
    if(delete==""):
        print(colored('Cancelled\n','red'))
        return
    servers.pop(int(delete))
    print(colored('Deleted\n','light_blue'))
    write_servers()

# Let's send some commands to the servers!
def send_commands():
    proceed=True # Used in while loops to determine if we still want to send more commands.
    print(colored('\n=== Send commands ===','green'))
    choice = input(colored('(Enter to cancel) Single (1) All (2)?: ','yellow'))
    if(choice==""):
        print(colored('Cancelled\n','red'))
        return
    if(choice=="1"): # We only want to interact with one server.
        print(colored(list_servers(2),'light_blue')) # Show us which servers we can pick from.
        server = input(colored('(Enter to cancel) Server?: ','yellow'))
        if(server==""):
            print(colored('Cancelled\n','red'))
            return
        server=int(server)
        while(proceed):
            command = input(colored('Input command (None to exit): ','yellow'))
            if(command!=""):
                with Client(servers[server]['address'], int(servers[server]['port']), passwd=servers[server]['password']) as client:
                    response = client.run(command)
                
                print(colored(servers[server]['name']+': '+response,'cyan'))
            else:
                proceed = False
        print(colored('No command was entered. Exiting.\n','red'))
    elif(choice=="2"):
        while(proceed):
            command = input(colored('Input command (None to exit): ','yellow'))
            for x in servers: # Run for each server.
                if(command!=""):
                    with Client(x['address'], int(x['port']), passwd=x['password']) as client:
                        response = client.run(command)
                    
                    print(colored(x['name']+': '+response,'cyan'))
                else:
                    proceed = False
        print(colored('No command was entered. Exiting.\n','red'))

# The main menu.
def initial_choice():
    print(colored('=== Choose Function ===\n1. Add servers\n2. Remove servers\n3. List servers\n4. Send commands\n5. Exit','green'))
    choose_func = input(colored("?: ",'yellow'))

    match choose_func:
            case "1":
                create_server()
                initial_choice()
            case "2":
                pop_server()
                initial_choice()
            case "3":
                print(colored(list_servers(),'light_blue'))
                initial_choice()
            case "4":
                send_commands()
                initial_choice()
            case "5":
                exit()                
            case _: 
                print(colored("\n!! Invalid input\n",'red'))
                initial_choice()

load_servers()
initial_choice()